@@ -1,5 +1,5 @@
 # Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: Chooser.pm,v 1.38 2007-08-05 18:47:57 nekral-guest Exp $
+# $Id: Chooser.pm,v 1.41 2008-07-20 16:31:55 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -36,6 +36,8 @@ sub new {
         $modname = 'LaTeX';
     } elsif ($module eq 'bibtex') {
         $modname = 'BibTex';
+    } elsif ($module eq 'tex') {
+        $modname = 'TeX';
     } else {
         $modname = ucfirst($module);
     }
@@ -69,7 +71,7 @@ sub list {
 	."\n  - ".gettext("texinfo: The info page format.")
 	."\n  - ".gettext("tex: generic TeX documents (see also latex).")
 	."\n  - ".gettext("text: simple text document.")
-#	."\n  - ".gettext("wml: WML documents.")
+	."\n  - ".gettext("wml: WML documents.")
 	."\n  - ".gettext("xhtml: XHTML documents.")
 	."\n  - ".gettext("xml: generic XML documents (see also docbook).")
     );
@@ -114,16 +116,20 @@ L<Locale::Po4a::Po(3pm)>
 L<Locale::Po4a::Dia(3pm)>,
 L<Locale::Po4a::Docbook(3pm)>,
 L<Locale::Po4a::Guide(3pm)>,
+L<Locale::Po4a::Halibut(3pm)>,
 L<Locale::Po4a::Ini(3pm)>,
 L<Locale::Po4a::KernelHelp(3pm)>,
 L<Locale::Po4a::LaTeX(3pm)>,
 L<Locale::Po4a::Man(3pm)>,
 L<Locale::Po4a::Pod(3pm)>,
 L<Locale::Po4a::Sgml(3pm)>,
+L<Locale::Po4a::TeX(3pm)>,
 L<Locale::Po4a::Texinfo(3pm)>,
 L<Locale::Po4a::Text(3pm)>,
+L<Locale::Po4a::Wml(3pm)>.
 L<Locale::Po4a::Xhtml(3pm)>,
-L<Locale::Po4a::Xml(3pm)>.
+L<Locale::Po4a::Xml(3pm)>,
+L<Locale::Po4a::Wml(3pm)>.
 
 =back
 
