@@ -89,34 +89,57 @@ sub initialize {
 	$self->{options}{'tagsonly'}=1;
 	$self->{options}{'wrap'}=1;
 	$self->{options}{'doctype'}=$self->{options}{'doctype'} || 'docbook xml';
-	my ($additional_tags, $additional_inline) = ("", "");
-	my $tag;
-	foreach $tag qw(
+	$self->{options}{'tags'} .= '
 		<abbrev>
 		<acronym>
+		W<address>
+		<affiliation>
+		<appendixinfo>
 		<arg>
 		<artheader>
+		<articleinfo>
 		<attribution>
+		<biblioentry>
+		<bibliographyinfo>
+		<blockinfo>
+		<bookinfo>
 		<citetitle>
+		<cmdsynopsis>
+		<confgroup>
+		<confdates>
+		<conftitle>
+		<confnum>
+		<confsponsor>
+		<contrib>
+		<chapterinfo>
+		<collab>
 		<computeroutput>
 		<date>
 		<entry>
 		<figure>
+		<glossaryinfo>
 		<glosssee>
 		<glossseealso>
 		<glossterm>
 		<holder>
+		<indexinfo>
+		<jobtitle>
+		<keyword>
 		<member>
 		<msgaud>
 		<msglevel>
 		<msgorig>
+		<objectinfo>
+		<orgdiv>
 		<option>
-		<orgname>
+		<othercredit>
 		<para>
 		<phrase>
+		<prefaceinfo>
+		<primary>
 		<pubdate>
 		<publishername>
-		<primary>
+		<pubsnumber>
 		W<programlisting>
 		<prompt>
 		<quote>
@@ -125,39 +148,47 @@ sub initialize {
 		<refmiscinfo>
 		<refname>
 		<refpurpose>
+		<refsynopsisdivinfo>
 		<releaseinfo>
 		<remark>
-		<replaceable>
+		<revision>
 		<revnumber>
+		<revremark>
 		W<screen>
 		<screeninfo>
+		<sect1info>
+		<sect2info>
+		<sect3info>
+		<sect4info>
+		<sect5info>
+		<sectioninfo>
 		<seg>
 		<segtitle>
+		<setinfo>
+		<shortaffil>
 		<simpara>
 		<subtitle>
 		<synopfragmentref>
+		<synopsis>
 		<term>
 		<title>
 		<titleabbrev>
-		<userinput>) {
-		if (not defined $self->{nodefault}{$tag}) {
-			$additional_tags .= " $tag";
-		}
-	}
-	foreach $tag qw(
+		<userinput>';
+	$self->{options}{'inline'} .= '
 		<action>
-		<affiliation>
 		<anchor>
 		<application>
 		<arg>
 		<author>
 		<authorinitials>
-		<command>
 		<citation>
 		<citerefentry>
 		<citetitle>
+		<city>
+		<country>
 		<classname>
 		<co>
+		<command>
 		<computeroutput>
 		<constant>
 		<corpauthor>
@@ -178,6 +209,7 @@ sub initialize {
 		<foreignphrase>
 		<function>
 		<glossterm>
+		<group>
 		<guibutton>
 		<guiicon>
 		<guilabel>
@@ -185,6 +217,8 @@ sub initialize {
 		<guimenuitem>
 		<guisubmenu>
 		<hardware>
+		<imageobject>
+		<imagedata>
 		<indexterm>
 		<informalexample>
 		<inlineequation>
@@ -211,10 +245,12 @@ sub initialize {
 		<ooexception>
 		<oointerface>
 		<optional>
-		<othercredit>
+		<orgname>
+		<othername>
 		<parameter>
 		<personname>
 		<phrase>
+		<postcode>
 		<productname>
 		<productnumber>
 		<prompt>
@@ -224,9 +260,10 @@ sub initialize {
 		<replaceable>
 		<remark>
 		<returnvalue>
-		<revhistory>
 		<sgmltag>
 		<sidebar>
+		<state>
+		<street>
 		<structfield>
 		<structname>
 		<subscript>
@@ -242,14 +279,9 @@ sub initialize {
 		<varname>
 		<wordasword>
 		<xref>
-		<year>) {
-		if (not defined $self->{nodefault}{$tag}) {
-			$additional_inline .= " $tag";
-		}
-	}
-	$self->{options}{'tags'} .= $additional_tags;
-	$self->{options}{'inline'} .= $additional_inline;
+		<year>';
 	$self->{options}{'attributes'}.='
 		lang';
+
 	$self->treat_options;
 }
