@@ -279,6 +279,40 @@ my box are inaccessible to po4a::man.
  .splitfont       .Sx             .T              .TF             .The
  .TT              .UC             .ul             .Vb             .zZ
 
+=head2 Hiding text to po4a
+
+Sometimes, the author knows that some parts are not translatable, and
+should not be extracted by po4a. For example, an option may accept an
+I<other> argument, and I<other> may also appear as the last item of a
+list. In the first case, I<other> should be not be translatable. And in
+the second case, I<other> should be translated.
+
+In such case, the author can avoid po4a to extract some strings, using
+some special groff constructs:
+
+ .if !'po4a'hide' .B other
+
+(this will require the B<-o groff_code=verbatim> option)
+
+A new macro can also be defined to automate this:
+ .de IR_untranslated
+ .    IR \\$@
+ ..
+
+ .IR_untranslated \-q ", " \-\-quiet
+
+(this will require the options B<-o groff_code=verbatim> and
+B<-o untranslated=IR_untranslated>; with this construct, the B<.if
+!'po4a'hide'> conditional is not strictly needed since po4a will not parse
+the internal of the macro definition)
+
+or using an alias:
+ .als IR_untranslated IR
+
+ .IR_untranslated \-q ", " \-\-quiet
+
+(this will require the B<-o untranslated=als,IR_untranslated> option)
+
 =head2 Conclusion
 
 To summarise this section, keep simple, and don't try to be clever while
@@ -350,7 +384,7 @@ L<Locale::Po4a::Pod(3pm)>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2002, 2003, 2004, 2005, 2006 by SPI, inc.
+Copyright 2002-2008 by SPI, inc.
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GPL (see the COPYING file).
@@ -615,38 +649,38 @@ NEW_LINE:
         my ($l, $t, $c) = ($1, $2, $3);
         $line = $l;
         unless ($allow_generated) {
-        # Check for comments indicating that the file was generated.
-        if ($c =~ /Pod::Man/) {
-            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the pod file with the pod module of po4a."));
-            exit 254;
-        } elsif ($c =~ /generated by help2man/)    {
-            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with help2man. Translate the source file with the regular gettext."));
-        } elsif ($c =~ /with docbook-to-man/)      {
-            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook-to-man. Translate the source file with the sgml module of po4a."));
-            exit 254;
-        } elsif ($c =~ /generated by docbook2man/) {
-            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook2man. Translate the source file with the sgml module of po4a."));
-            exit 254;
-        } elsif ($c =~ /created with latex2man/)   {
-            warn wrap_mod("po4a::man", dgettext("po4a",
-                "This file was generated with %s. ".
-                "You should translate the source file, but continuing anyway."
-                ),"latex2man");
-        } elsif ($c =~ /Generated by db2man.xsl/)  { 
-            warn wrap_mod("po4a::man", dgettext("po4a","This file was generated with db2man.xsl. Translate the source file with the xml module of po4a."));
-            exit 254;
-        } elsif ($c =~ /generated automatically by mtex2man/)  {
-            warn wrap_mod("po4a::man", dgettext("po4a",
-                "This file was generated with %s. ".
-                "You should translate the source file, but continuing anyway."
-                ),"mtex2man");
-        } elsif ($c =~ /THIS FILE HAS BEEN AUTOMATICALLY GENERATED.  DO NOT EDIT./ ||
-                 $c =~ /DO NOT EDIT/i || $c =~ /generated/i) {
-            warn wrap_mod("po4a::man", dgettext("po4a",
-                "This file contains the line '%s'. ".
-                "You should translate the source file, but continuing anyway."
-                ),$l."\\\"".$c);
-        }
+                # Check for comments indicating that the file was generated.
+                if ($c =~ /Pod::Man/) {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the pod file with the pod module of po4a."));
+                    exit 254;
+                } elsif ($c =~ /generated by help2man/)    {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with help2man. Translate the source file with the regular gettext."));
+                } elsif ($c =~ /with docbook-to-man/)      {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with docbook-to-man. Translate the source file with the sgml module of po4a."));
+                    exit 254;
+                } elsif ($c =~ /generated by docbook2man/) {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with docbook2man. Translate the source file with the sgml module of po4a."));
+                    exit 254;
+                } elsif ($c =~ /created with latex2man/)   {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+                        "This file was generated with %s. ".
+                        "You should translate the source file, but continuing anyway."
+                        ),"latex2man");
+                } elsif ($c =~ /Generated by db2man.xsl/)  { 
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a","This file was generated with db2man.xsl. Translate the source file with the xml module of po4a."));
+                    exit 254;
+                } elsif ($c =~ /generated automatically by mtex2man/)  {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+                        "This file was generated with %s. ".
+                        "You should translate the source file, but continuing anyway."
+                        ),"mtex2man");
+                } elsif ($c =~ /THIS FILE HAS BEEN AUTOMATICALLY GENERATED.  DO NOT EDIT./ ||
+                         $c =~ /DO NOT EDIT/i || $c =~ /generated/i) {
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+                        "This file contains the line '%s'. ".
+                        "You should translate the source file, but continuing anyway."
+                        ),$l."\\\"".$c);
+                }
         }
 
         if ($line =~ m/^[.']*$/) {
@@ -974,18 +1008,21 @@ sub post_trans {
             $str = $2;
             my $tmp2 = $tmp.$begin;
             if (   ($begin =~ m/(?<!\\)(\\\\)*\\s$/s)
-                or ($begin =~ m/(?<!\\)(\\\\)*\\\((.|E<[gl]t>)$/s)
-                or ($tmp2 =~ m/(?<!\\)(\\\\)*\\[hCv]'([^']|(?<!\\)(\\\\)*\\')*$/)
-                or ($tmp2 =~ m/(?<!\\)(\\\\)*\\\[([^\]]|(?<!\\)(\\\\)*\\\[)*$/)
+                or ($begin =~ m/(?<!\\)(\\\\)*\\\((.|E<[gl]t>)?$/s)
+                or ($tmp2 =~ m/(?<!\\)(\\\\)*\\[ZHhCv]'([^']|(?<!\\)(\\\\)*\\')*$/)
+                or ($tmp2 =~ m/(?<!\\)(\\\\)*\\(\*)?\[([^\]]|(?<!\\)(\\\\)*\\\[)*$/)
                 or ($tmp2 =~ m/(?<!\\)(\\\\)*\\\*\(.?$/)) {
                 # Do not change - to \- for
                 #  * \s-n (reduce font size)
-                #  * \(.- (a character named '.-', e.g. '<-')
+                #  * \(.. (e.g. '<-', '-D')
                 #  * inside a \h'...'
                 #  * inside a \C'...'
                 #  * inside a \[...]
                 #  * inside a \*(..
+                #  * inside a \*[...]
                 #  * inside a \v'...'
+                #  * inside a \H'...'
+                #  * inside a \Z'...'
                 $tmp = $tmp2."-";
             } else {
                 $tmp = $tmp2."\\-";
@@ -996,7 +1033,7 @@ sub post_trans {
     }
 
     # There must not be an end of line inside an inline macro
-    $str =~ s/(E<\.[^>]*)\n([^>]*>)/$1$2/gs;
+    $str =~ s/(E<\.[^>]*)\n([^>]*>)/$1 $2/gs;
 
     # No . or ' on first char, or nroff will think it's a macro
     # * at the beginning of a paragraph, add \& (zero width space) at
@@ -1079,6 +1116,12 @@ sub post_trans {
             $str.= "\n$t3";
         }
     }
+    my $str2 = $str;
+    $str2 =~ s/E<[gl]t>//g;
+    die wrap_ref_mod($ref||$self->{ref}, "po4a::man",
+                     dgettext("po4a","Unknown '<' or '>' sequence. ".
+                                     "Faulty message: %s"),$str)
+        if $str2 =~ /[<>]/;
     $str =~ s/E<gt>/>/mg;
     $str =~ s/E<lt>/</mg;
     # Don't do that, because we'll go into trouble if previous line was .TP
@@ -2001,8 +2044,8 @@ $macro{'bp'}=\&untranslated;
 $macro{'ad'}=\&untranslated;
 # .de macro Define or redefine macro until .. is encountered.
 $macro{'de'}=sub {
+    my $self = shift;
     if ($groff_code ne "fail") {
-        my $self = shift;
         my $paragraph = "@_";
         my $end = ".";
         if ($paragraph=~/^[.'][\t ]*de[\t ]+([^\t ]+)[\t ]+([^\t ]+)[\t ]$/) {
@@ -2028,7 +2071,7 @@ $macro{'de'}=sub {
                                               "wrap" => 0) );
         }
     } else {
-        die wrap_mod("po4a::man", dgettext("po4a", "This page defines a new macro with '.de'. Since po4a is not a real groff parser, this is not supported."));
+        die wrap_ref_mod($self->{ref}, "po4a::man", dgettext("po4a", "This page defines a new macro with '.de'. Since po4a is not a real groff parser, this is not supported."));
     }
 };
 # .ds stringvar anything
@@ -2435,7 +2478,7 @@ sub define_mdoc_macros {
     foreach (qw(Ad An Ar Cd Cm Dv Er Ev Fa Fd Fn Ic Li Nm Op Ot Pa St Va Vt Xr)) {
         $inline{$_} = 1;
     }
-    # FIXME: some of thes macro introduce a line in bold.
+    # FIXME: some of these macros introduce a line in bold.
     #        Using \fP in these line is not supported.
     #        do_fonts should be called for every inline line
 
