@@ -42,6 +42,10 @@ areas where they were not expected like documentation.
 Locale::Po4a::Sgml is a module to help the translation of documentation in
 the SGML format into other [human] languages.
 
+This module uses B<nsgmls> to parse the SGML files. Make sure it is
+installed.
+Also make sure that the DTD of the SGML files are installed in the system.
+
 =head1 OPTIONS ACCEPTED BY THIS MODULE
 
 =over 4
@@ -105,7 +109,8 @@ given attribute into the 'attributes' list too.
 
 =item force
 
-Proceed even if the DTD is unknown.
+Proceed even if the DTD is unknown or if nsgmls finds errors in the input
+file.
 
 =item include-all
 
@@ -363,6 +368,26 @@ sub parse_file {
         $origfile .= ${$self->{TT}{doc_in}}[$i];
         $i+=2;
     }
+
+    unless ($self->{options}{'force'}) {
+        # Detect if we can find the DTD
+        my ($tmpfh,$tmpfile)=File::Temp::tempfile("po4a-XXXX",
+                                                  SUFFIX => ".sgml",
+                                                  DIR    => "/tmp",
+                                                  UNLINK => 0);
+        print $tmpfh $origfile;
+        close $tmpfh
+            or die wrap_mod("po4a::sgml",
+                            dgettext("po4a", "Can't close tempfile: %s"), $!);
+        if (system("nsgmls -p < $tmpfile")) {
+            unlink ($tmpfile);
+            die wrap_mod("po4a::sgml",
+                         dgettext("po4a", "Error while running nsgmls -p.  ".
+                                          "Please check if nsgmls and the ".
+                                          "DTD are installed."));
+        }
+        unlink ($tmpfile);
+    }
     # Detect the XML pre-prolog
     if ($origfile =~ s/^(\s*<\?xml[^?]*\?>)//) {
 	warn wrap_mod("po4a::sgml", dgettext("po4a",
