@@ -77,6 +77,7 @@ use vars qw(@ISA @EXPORT);
              $verbatim_environments
              %separated_command
              %separated_environment
+             %translate_buffer_env
              &generic_command
              &register_generic_command
              &register_generic_environment);
@@ -468,7 +469,7 @@ sub get_leading_command {
     print STDERR "get_leading_command($buffer)="
         if ($debug{'extract_commands'});
 
-    if ($buffer =~ m/^$RE_ESCAPE([[:alpha:]]+)(\*?)(.*)$/s
+    if ($buffer =~ m/^$RE_ESCAPE([[:alnum:]]+)(\*?)(.*)$/s
         && defined $separated_command{$1}) {
         # The buffer begin by a comand (possibly preceded by some
         # whitespaces).
@@ -587,7 +588,7 @@ sub get_trailing_command {
     }
 
     # There should now be a command, maybe followed by an asterisk.
-    if ($buffer =~ m/^(.*)$RE_ESCAPE([[:alpha:]]+)(\*?)\s*$/s
+    if ($buffer =~ m/^(.*)$RE_ESCAPE([[:alnum:]]+)(\*?)\s*$/s
         && defined $separated_command{$2}) {
         $buffer = $1;
         $command = $2;
@@ -627,12 +628,23 @@ Recursively translate a buffer by separating leading and trailing
 commands (those which should be translated separately) from the
 buffer.
 
+If a function is defined in %translate_buffer_env for the current
+environment, this function will be used to translate the buffer instead of
+translate_buffer().
+
 =cut
 
+our %translate_buffer_env = ();
 sub translate_buffer {
-    my ($self,$buffer,@env) = (shift,shift,@_);
-    print STDERR "translate_buffer($buffer,@env)="
+    my ($self,$buffer,$no_wrap,@env) = (shift,shift,shift,@_);
+
+    if (@env and defined $translate_buffer_env{$env[-1]}) {
+        return &{$translate_buffer_env{$env[-1]}}($self,$buffer,$no_wrap,@env);
+    }
+
+    print STDERR "translate_buffer($buffer,$no_wrap,@env)="
         if ($debug{'translate_buffer'});
+
     my ($command,$variant) = ("","");
     my $args;
     my $translated_buffer = "";
@@ -656,8 +668,8 @@ sub translate_buffer {
                                        $env[-1],
                                        "wrap" => 0);
             }
-            ($t2, @env) = translate_buffer($self, $end, @env);
-            print STDERR "($t1.$t2,@env)\n"
+            ($t2, @env) = translate_buffer($self, $end, $no_wrap, @env);
+            print STDERR "($t1$t2,@env)\n"
                 if ($debug{'translate_buffer'});
             return ($t1.$t2, @env);
         } else {
@@ -673,10 +685,10 @@ sub translate_buffer {
     if ($buffer =~ /^($RE_VERBATIM\n?)(.*)$/s and length $2) {
         my ($begin, $end) = ($1, $2);
         my ($t1, $t2) = ("", "");
-        ($t1, @env) = translate_buffer($self, $begin, @env);
-        ($t2, @env) = translate_buffer($self, $end,   @env);
+        ($t1, @env) = translate_buffer($self, $begin, $no_wrap, @env);
+        ($t2, @env) = translate_buffer($self, $end,   $no_wrap, @env);
 
-        print STDERR "($t1.$t2,@env)\n"
+        print STDERR "($t1$t2,@env)\n"
             if ($debug{'translate_buffer'});
         return ($t1.$t2, @env);
     }
@@ -691,10 +703,10 @@ sub translate_buffer {
         my ($begin, $end) = ($1, $2);
         my ($t1, $t2) = ("", "");
         if (is_closed($begin)) {
-            ($t1, @env) = translate_buffer($self, $begin, @env);
-            ($t2, @env) = translate_buffer($self, $end,   @env);
+            ($t1, @env) = translate_buffer($self, $begin, $no_wrap, @env);
+            ($t2, @env) = translate_buffer($self, $end,   $no_wrap, @env);
 
-            print STDERR "($t1.$t2,@env)\n"
+            print STDERR "($t1$t2,@env)\n"
                 if ($debug{'translate_buffer'});
             return ($t1.$t2, @env);
         }
@@ -726,7 +738,7 @@ sub translate_buffer {
         if ($buffer =~ /^(\s+)(.*?)$/s) {
             $spaces = $1;
 #            $buffer = $2; # FIXME: this also remove trailing spaces!!
-            $buffer =~ s/^\s*//;
+            $buffer =~ s/^\s*//s;
         }
         my $buffer_save = $buffer;
         ($command, $variant, $args, $buffer) =
@@ -748,14 +760,14 @@ sub translate_buffer {
             # with the content of each argument that need a translation.
             if (defined ($commands{$command})) {
                 ($t,@env) = &{$commands{$command}}($self,$command,$variant,
-                                                   $args,\@env);
+                                                   $args,\@env,$no_wrap);
                 $translated_buffer .= $spaces.$t;
                 # Handle spaces after a command.
                 $spaces = "";
                 if ($buffer =~ /^(\s+)(.*?)$/s) {
                     $spaces = $1;
 #                    $buffer = $2;  # FIXME: this also remove trailing spaces!!
-                    $buffer =~ s/^\s*//;
+                    $buffer =~ s/^\s*//s;
                 }
                 $translated_buffer .= $spaces;
             } else {
@@ -808,7 +820,7 @@ sub translate_buffer {
             $buf_begin .= $begin;
             if (is_closed($buf_begin)) {
                 my $t = "";
-                ($t, @env) = translate_buffer($self, $buf_begin, @env);
+                ($t, @env) = translate_buffer($self,$buf_begin,$no_wrap,@env);
                 $translated_buffer .= $t.$sep;
                 $buf_begin = "";
             } else {
@@ -832,17 +844,21 @@ sub translate_buffer {
                 }
             }
         }
+        $wrap = 0 if (defined $no_wrap and $no_wrap == 1);
         # Keep spaces at the end of the buffer.
-        my $spaces = "";
+        my $spaces_end = "";
         if ($buffer =~ /^(.*?)(\s+)$/s) {
-            $spaces = $2;
+            $spaces_end = $2;
             $buffer = $1;
         }
+        if ($wrap and $buffer =~ s/^(\s+)//s) {
+                $translated_buffer .= $1;
+        }
         $translated_buffer .= $self->translate($buffer,$self->{ref},
                                                @env?$env[-1]:"Plain text",
                                                "wrap" => $wrap);
         # Restore spaces at the end of the buffer.
-        $translated_buffer .= $spaces;
+        $translated_buffer .= $spaces_end;
     }
 
     # append the translation of the trailing commands
@@ -853,7 +869,7 @@ sub translate_buffer {
         my $spaces  = shift @trailing_commands;
         if (defined ($commands{$command})) {
             ($t,@env) = &{$commands{$command}}($self,$command,$variant,
-                                               $args,\@env);
+                                               $args,\@env,$no_wrap);
             $translated_buffer .= $t.$spaces;
         } else {
             die wrap_ref_mod($self->{ref},
@@ -1159,7 +1175,7 @@ sub parse {
             # paragraph.
             $paragraph .= $line."\n";
             if (length($paragraph)) {
-                ($t, @env) = translate_buffer($self,$paragraph,@env);
+                ($t, @env) = translate_buffer($self,$paragraph,undef,@env);
                 $self->pushline($t);
                 $paragraph="";
                 @comments = ();
@@ -1175,7 +1191,7 @@ sub parse {
     }
 
     if (length($paragraph)) {
-        ($t, @env) = translate_buffer($self,$paragraph,@env);
+        ($t, @env) = translate_buffer($self,$paragraph,undef,@env);
         $self->pushline($t);
         $paragraph="";
     }
@@ -1247,7 +1263,8 @@ followed by an identifier of the parameter (like {#7} or [#2]).
 $commands{'begin'}= sub {
     my $self = shift;
     my ($command,$variant,$args,$env) = (shift,shift,shift,shift);
-    print "begin($command,$variant,@$args,@$env)="
+    my $no_wrap = shift;
+    print "begin($command,$variant,@$args,@$env,$no_wrap)="
         if ($debug{'commands'} || $debug{'environments'});
     my ($t,@e) = ("",());
 
@@ -1258,7 +1275,7 @@ $commands{'begin'}= sub {
 
     if (defined($envir) && defined($environments{$envir})) {
         ($t, @e) = &{$environments{$envir}}($self,$command,$variant,
-                                            $args,$env);
+                                            $args,$env,$no_wrap);
     } else {
         die wrap_ref_mod($self->{ref}, "po4a::tex",
                      dgettext("po4a", "unknown environment: '%s'"),
@@ -1275,7 +1292,8 @@ register_generic_command("*end,{}");
 $commands{'end'}= sub {
     my $self = shift;
     my ($command,$variant,$args,$env) = (shift,shift,shift,shift);
-    print "end($command,$variant,@$args,@$env)="
+    my $no_wrap = shift;
+    print "end($command,$variant,@$args,@$env,$no_wrap)="
         if ($debug{'commands'} || $debug{'environments'});
 
     # verify that this environment was the last pushed environment.
@@ -1289,7 +1307,7 @@ $commands{'end'}= sub {
         pop @$env;
     }
 
-    my ($t,@e) = generic_command($self,$command,$variant,$args,$env);
+    my ($t,@e) = generic_command($self,$command,$variant,$args,$env,$no_wrap);
 
     print "($t, @$env)\n"
         if ($debug{'commands'} || $debug{'environments'});
@@ -1300,8 +1318,10 @@ $separated_command{'begin'} = '*';
 sub generic_command {
     my $self = shift;
     my ($command,$variant,$args,$env) = (shift,shift,shift,shift);
-    print "generic_command($command,$variant,@$args,@$env)="
+    my $no_wrap = shift;
+    print "generic_command($command,$variant,@$args,@$env,$no_wrap)="
         if ($debug{'commands'} || $debug{'environments'});
+
     my ($t,@e)=("",());
     my $translated = "";
 
@@ -1350,7 +1370,7 @@ TEST_TYPE:
                                  $command, $reason);
             }
             if ($have_to_be_translated) {
-                ($t, @e) = translate_buffer($self,$opt,(@$env,$command.$type."#".$count.$type_end{$type}));
+                ($t, @e) = translate_buffer($self,$opt,$no_wrap,(@$env,$command.$type."#".$count.$type_end{$type}));
             } else {
                 $t = $opt;
             }
@@ -1366,9 +1386,11 @@ TEST_TYPE:
             $tmp .= $type.$opt.$type_end{$type};
         }
         @e = @$env;
+        my $wrap = 1;
+        $wrap = 0 if (defined $no_wrap and $no_wrap == 1);
         $translated = $self->translate($tmp,$self->{ref},
                                        @e?$e[-1]:"Plain text",
-                                       "wrap" => 1);
+                                       "wrap" => $wrap);
     }
 
     print "($translated, @$env)\n"
@@ -1412,7 +1434,8 @@ sub register_generic_command {
 sub generic_environment {
     my $self = shift;
     my ($command,$variant,$args,$env) = (shift,shift,shift,shift);
-    print "generic_environment($command,$variant,$args,$env)="
+    my $no_wrap = shift;
+    print "generic_environment($command,$variant,$args,$env,$no_wrap)="
         if ($debug{'environments'});
     my ($t,@e)=("",());
     my $translated = "";
@@ -1463,7 +1486,7 @@ TEST_TYPE:
             }
 
             if ($have_to_be_translated) {
-                ($t, @e) = translate_buffer($self,$opt,(@$env,$new_env.$type."#".$count.$type_end{$type}));
+                ($t, @e) = translate_buffer($self,$opt,$no_wrap,(@$env,$new_env.$type."#".$count.$type_end{$type}));
             } else {
                 $t = $opt;
             }
@@ -1480,9 +1503,11 @@ TEST_TYPE:
             $buf .= $type.$opt.$type_end{$type};
         }
         @e = @$env;
+        my $wrap = 1;
+        $wrap = 0 if $no_wrap == 1;
         $translated = $self->translate($buf,$self->{ref},
                                        @e?$e[-1]:"Plain text",
-                                       "wrap" => 1);
+                                       "wrap" => $wrap);
     }
     @e = (@$env, $new_env);
 
