@@ -83,6 +83,23 @@ released versions, which only contain non translatable informations.
 
 my $debianchangelog = 0;
 
+=item B<fortunes>
+
+Handle the fortunes format, which separate fortunes with a line which
+consists in '%' or '%%', and use '%%' as the beginning of a comment.
+
+=cut
+
+my $fortunes = 0;
+
+=item B<markdown>
+
+Handle some special markup in Markdown-formatted texts.
+
+=cut
+
+my $markdown = 0;
+
 sub initialize {
     my $self = shift;
     my %options = @_;
@@ -96,6 +113,14 @@ sub initialize {
     if (defined $options{'debianchangelog'}) {
         $debianchangelog=1;
     }
+
+    if (defined $options{'fortunes'}) {
+        $fortunes=1;
+    }
+
+    if (defined $options{'markdown'}) {
+        $markdown=1;
+    }
 }
 
 sub parse {
@@ -105,7 +130,16 @@ sub parse {
     my $wrapped_mode = 1;
     my $expect_header = 1;
     ($line,$ref)=$self->shiftline();
+    my $file = $ref;
+    $file =~ s/:[0-9]+$//;
     while (defined($line)) {
+        $ref =~ m/^(.*):[0-9]+$/;
+        if ($1 ne $file) {
+            $file = $1;
+            do_paragraph($self,$paragraph,$wrapped_mode);
+            $paragraph="";
+        }
+
         chomp($line);
         $self->{ref}="$ref";
         if ($debianchangelog and
@@ -124,6 +158,15 @@ sub parse {
             $paragraph="";
             $self->pushline("$line\n");
             $expect_header=1;
+        } elsif ($fortunes and
+                 $line =~ m/^%%?\s*$/) {
+            # Found end of fortune
+            do_paragraph($self,$paragraph,$wrapped_mode);
+            $self->pushline("\n") unless (   $wrapped_mode == 0
+                                          or $paragraph eq "");
+            $paragraph="";
+            $wrapped_mode = 1;
+            $self->pushline("$line\n");
         } elsif ($line =~ /^\s*$/) {
             # Break paragraphs on lines containing only spaces
             do_paragraph($self,$paragraph,$wrapped_mode);
@@ -140,12 +183,34 @@ sub parse {
             do_paragraph($self,$paragraph,$wrapped_mode);
             $paragraph="";
             $wrapped_mode = 1;
+        } elsif ($markdown and
+                 (   $line =~ /^#/            # headline
+                  or $line =~ /^\s*\[\[\!\S[^\]]*\]\]\s*$/)) { # sole macro
+            # Found Markdown markup that should be preserved as a single line
+            do_paragraph($self,$paragraph,$wrapped_mode);
+            $paragraph="$line\n";
+            $wrapped_mode = 0;
+            do_paragraph($self,$paragraph,$wrapped_mode);
+            $wrapped_mode = 1;
+            $paragraph="";
+        } elsif ($markdown and
+                 (   $paragraph =~ m/^>/       # blockquote
+                  or $paragraph =~ m/[<>]/     # maybe html
+                  or $paragraph =~ m/^"""/     # textblock inside macro end
+                  or $paragraph =~ m/"""$/)) { # textblock inside macro begin
+            # Found Markdown markup that might not survive wrapping
+            $wrapped_mode = 0;
+            $paragraph .= $line."\n";
         } else {
             if ($line =~ /^\s/) {
                 # A line starting by a space indicates a non-wrap
                 # paragraph
                 $wrapped_mode = 0;
             }
+            if ($fortunes) {
+                $line =~ s/%%(.*)$//;
+            }
+# TODO: comments
             $paragraph .= $line."\n";
         }
         # paragraphs starting by a bullet, or numbered
@@ -230,7 +295,7 @@ Tested successfully on simple text files and NEWS.Debian files.
 
 =head1 COPYRIGHT AND LICENSE
 
- Copyright 2005,2007 by Nicolas FRANÇOIS <nicolas.francois@centraliens.net>.
+ Copyright 2005-2008 by Nicolas FRANÇOIS <nicolas.francois@centraliens.net>.
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GPL (see the COPYING file).
