@@ -1,4 +1,6 @@
 #!/usr/bin/perl
+#TODO:
+#Warn if tags / tagsonly / inline is used
 
 # Po4a::Xml.pm 
 # 
@@ -218,7 +220,8 @@ documents.
 =item ontagerror
 
 This option defines the behavior of the module when it encounter a invalid
-closing tag (a tag is closed, which does not match the last opening tag).
+Xml syntax (a closing tag which does not match the last opening tag, or a
+tag's attribute without value).
 It can take the following values:
 
 =over
@@ -246,6 +249,8 @@ It is generally recommended to fix the input file.
 Extracts only the specified tags in the "tags" option.  Otherwise, it
 will extract all the tags except the ones specified.
 
+Note: This option is deprecated.
+
 =item doctype
 
 String that will try to match with the first line of the document's doctype
@@ -265,6 +270,9 @@ to override the default behavior specified by the global "wrap" option.
 
 Example: WE<lt>chapterE<gt>E<lt>titleE<gt>
 
+Note: This option is deprecated.
+You should use the B<translated> and B<untranslate> options instead.
+
 =item attributes
 
 Space-separated list of the tag's attributes you want to translate.  You can
@@ -284,6 +292,33 @@ all tags break the sequence.  This follows the same syntax as the tags option.
 Space separated list of tags that the module should not try to set by
 default in the "tags" or "inline" category.
 
+=item cpp
+
+Support C preprocessor directives.
+When this option is set, po4a will consider preprocessor directives as
+paragraph separators.
+This is important if the XML file must be preprocessed because otherwise
+the directives may be inserted in the middle of lines if po4a consider it
+belong to the current paragraph, and they won't be recognized by the
+preprocessor.
+Note: the preprocessor directives must only appear between tags
+(they must not break a tag).
+
+=item translated
+
+=item unstranslated
+
+Space-separated list of the tags you want to translate or not.
+The tags must be in the form <aaa>, but you can join some (<bbb><aaa>) to
+indicate that the content of the tag <aaa> will only be translated when
+it’s into a <bbb> tag.
+
+You can also specify some tag options putting some characters in front of
+the tag hierarchy. For example, you can put ’w’ (wrap) or ’W’ (don’t wrap)
+to overide the default behavior specified by the global "wrap" option.
+
+Example: WE<lt>chapterE<gt>E<lt>titleE<gt>
+
 =back
 
 =cut
@@ -305,6 +340,9 @@ sub initialize {
 	$self->{options}{'caseinsensitive'}=0;
 	$self->{options}{'tagsonly'}=0;
 	$self->{options}{'tags'}='';
+	$self->{options}{'translated'}='';
+	$self->{options}{'untranslated'}='';
+	$self->{options}{'defaulttranslateoption'}='';
 	$self->{options}{'attributes'}='';
 	$self->{options}{'inline'}='';
 	$self->{options}{'placeholder'}='';
@@ -312,6 +350,7 @@ sub initialize {
 	$self->{options}{'nodefault'}='';
 	$self->{options}{'includeexternal'}=0;
 	$self->{options}{'ontagerror'}="fail";
+	$self->{options}{'cpp'}=0;
 
 	$self->{options}{'verbose'}='';
 	$self->{options}{'debug'}='';
@@ -327,6 +366,8 @@ sub initialize {
 
 	#It will maintain the list of the translatable tags
 	$self->{tags}=();
+	$self->{translated}=();
+	$self->{untranslated}=();
 	#It will maintain the list of the translatable attributes
 	$self->{attributes}=();
 	#It will maintain the list of the inline tags
@@ -404,6 +445,10 @@ some strings.
 sub found_string {
 	my ($self,$text,$ref,$options)=@_;
 
+	if ($text =~ m/^\s*$/s) {
+		return $text;
+	}
+
 	my $comment;
 	my $wrap = $self->{options}{'wrap'};
 
@@ -482,7 +527,9 @@ single string.
 our @tag_types = ( 
 	{	beginning	=> "!--#",
 		end		=> "--",
-		breaking	=> 0},
+		breaking	=> 0,
+		f_extract	=> \&tag_extract_comment,
+		f_translate	=> \&tag_trans_comment},
 	{	beginning	=> "!--",
 		end		=> "--",
 		breaking	=> 0,
@@ -643,8 +690,8 @@ sub tag_trans_doctype {
 
 sub tag_break_close {
 	my ($self,@tag)=@_;
-	if ($self->tag_in_list($self->get_path."<".
-		$self->get_tag_name(@tag).">",@{$self->{inline}})) {
+	my $struct = $self->get_path."<".$self->get_tag_name(@tag).">";
+	if ($self->get_translate_options($struct) =~ m/i/) {
 		return 0;
 	} else {
 		return 1;
@@ -683,8 +730,8 @@ sub CDATA_trans {
 
 sub tag_break_alone {
 	my ($self,@tag)=@_;
-	if ($self->tag_in_list($self->get_path."<".
-		$self->get_tag_name(@tag).">",@{$self->{inline}})) {
+	my $struct = $self->get_path."<".$self->get_tag_name(@tag).">";
+	if ($self->get_translate_options($struct) =~ m/i/) {
 		return 0;
 	} else {
 		return 1;
@@ -704,8 +751,8 @@ sub tag_trans_alone {
 
 sub tag_break_open {
 	my ($self,@tag)=@_;
-	if ($self->tag_in_list($self->get_path."<".
-		$self->get_tag_name(@tag).">",@{$self->{inline}})) {
+	my $struct = $self->get_path."<".$self->get_tag_name(@tag).">";
+	if ($self->get_translate_options($struct) =~ m/i/) {
 		return 0;
 	} else {
 		return 1;
@@ -1010,35 +1057,137 @@ sub treat_attributes {
 				}
 			}
           
-			die wrap_ref_mod($ref, "po4a::xml", dgettext ("po4a", "Bad attribute syntax"))
-				unless ($complete);
+			unless ($complete) {
+				my $ontagerror = $self->{options}{'ontagerror'};
+				if ($ontagerror eq "warn") {
+					warn wrap_ref_mod($ref, "po4a::xml", dgettext ("po4a", "Bad attribute syntax.  Continuing..."));
+				} elsif ($ontagerror ne "silent") {
+					die wrap_ref_mod($ref, "po4a::xml", dgettext ("po4a", "Bad attribute syntax"));
+				}
+			}
 		}
 	}
 	return $text;
 }
 
+# Returns an empty string if the content in the $path should not be
+# translated.
+#
+# Otherwise, returns the set of options for translation:
+#   w: the content shall be re-wrapped
+#   W: the content shall not be re-wrapped
+#   i: the tag shall be inlined
+sub get_translate_options {
+	my $self = shift;
+	my $path = shift;
+	my $options = "";
+	my $translate = 0;
+	my $usedefault = 1;
+
+	my $inlist = 0;
+	my $tag = $self->get_tag_from_list($path, @{$self->{tags}});
+	if (defined $tag) {
+		$inlist = 1;
+	}
+	if ($self->{options}{'tagsonly'} eq $inlist) {
+		$usedefault = 0;
+		if (defined $tag) {
+			$options = $tag;
+			$options =~ s/<.*$//;
+		} else {
+			if ($self->{options}{'wrap'}) {
+				$options = "w";
+			} else {
+				$options = "W";
+			}
+		}
+		$translate = 1;
+	}
+
+	$tag = $self->get_tag_from_list($path, @{$self->{inline}});
+	if (defined $tag) {
+		$usedefault = 0;
+		$options .= "i";
+	}
+
+# TODO: a less precise set of tags should not override a more precise one
+	# The tags and tagsonly options are deprecated.
+	# The translated and untranslated options have an higher priority.
+	$tag = $self->get_tag_from_list($path, @{$self->{translated}});
+	if (defined $tag) {
+		$usedefault = 0;
+		$options = $tag;
+		$options =~ s/<.*$//;
+		$translate = 1;
+	}
+
+	if ($translate and $options !~ m/w/i) {
+		$options .= ($self->{options}{'wrap'})?"w":"W";
+	}
+
+	$tag = $self->get_tag_from_list($path, @{$self->{untranslated}});
+	if (defined $tag) {
+		$usedefault = 0;
+		$options = "";
+		$translate = 0;
+	}
+
+	if ($usedefault) {
+		$options = $self->{options}{'defaulttranslateoption'};
+	}
+
+	return $options;
+}
+
+
+# Return the tag (or biggest set of tags) of a list which matches with the
+# given path.
+#
+# The tag (or set of tags) is returned with its options.
+#
+# If no tags could match the path, undef is returned.
+sub get_tag_from_list {
+	my ($self,$path,@list) = @_;
+	my $found = 0;
+	my $i = 0;
+	
+	while (!$found && $i < @list) {
+		my $options;
+		my $element;
+		if ($list[$i] =~ /(.*?)(<.*)/) {
+			$options = $1;
+			$element = $2;
+		} else {
+			$element = $list[$i];
+		}
+		if ($self->{options}{'caseinsensitive'}) {
+			if ( $path =~ /\Q$element\E$/i ) {
+				$found = 1;
+			}
+		} else {
+			if ( $path =~ /\Q$element\E$/ ) {
+				$found = 1;
+			}
+		}
+		if ($found) {
+			return $list[$i];
+		} else {
+			$i++;
+		}
+	}
+	return undef;
+}
+
 
 
 sub treat_content {
 	my $self = shift;
 	my $blank="";
 	# Indicates if the paragraph will have to be translated
-	my $translate = 0;
+	my $translate = "";
 
 	my ($eof,@paragraph)=$self->get_string_until('<',{remove=>1});
 
-	# Check if this has to be translated
-	if ($self->join_lines(@paragraph) !~ /^\s*$/s) {
-		my $struc = $self->get_path;
-		my $inlist = 0;
-		if ($self->tag_in_list($struc,@{$self->{tags}})) {
-			$inlist = 1;
-		}
-		if ($self->{options}{'tagsonly'} eq $inlist) {
-			$translate = 1;
-		}
-	}
-
 	while (!$eof and !$self->breaking_tag) {
 	NEXT_TAG:
 		my @text;
@@ -1048,6 +1197,10 @@ sub treat_content {
 		    and $f_extract eq \&tag_extract_comment) {
 			# Remove the content of the comments
 			($eof, @text) = $self->extract_tag($type,1);
+			$text[$#text-1] .= "\0";
+			if ($tag_types[$type]->{'beginning'} eq "!--#") {
+				$text[0] = "#".$text[0];
+			}
 			push @comments, @text;
 		} else {
 			my ($tmpeof, @tag) = $self->extract_tag($type,0);
@@ -1099,13 +1252,14 @@ sub treat_content {
 					# Check if this is closing the
 					# last opening tag we detected.
 					my $test = pop @path;
+					my $name = $self->get_tag_name(@tag);
 					if (!defined($test) ||
-					    $test ne $tag[0] ) {
+					    $test ne $name ) {
 						my $ontagerror = $self->{options}{'ontagerror'};
 						if ($ontagerror eq "warn") {
-							warn wrap_ref_mod($tag[1], "po4a::xml", dgettext("po4a", "Unexpected closing tag </%s> found. The main document may be wrong.  Continuing..."), $tag[0]);
+							warn wrap_ref_mod($tag[1], "po4a::xml", dgettext("po4a", "Unexpected closing tag </%s> found. The main document may be wrong.  Continuing..."), $name);
 						} elsif ($ontagerror ne "silent") {
-							die wrap_ref_mod($tag[1], "po4a::xml", dgettext("po4a", "Unexpected closing tag </%s> found. The main document may be wrong."), $tag[0]);
+							die wrap_ref_mod($tag[1], "po4a::xml", dgettext("po4a", "Unexpected closing tag </%s> found. The main document may be wrong."), $name);
 						}
 					}
 
@@ -1155,17 +1309,6 @@ sub treat_content {
 		if ($#text > 0) {
 			# Check if text (extracted after the inline tag)
 			# has to be translated
-			if ($self->join_lines(@text) !~ /^\s*$/s) {
-				my $struc = $self->get_path;
-				my $inlist = 0;
-				if ($self->tag_in_list($struc,
-				                       @{$self->{tags}})) {
-					$inlist = 1;
-				}
-				if ($self->{options}{'tagsonly'} eq $inlist) {
-					$translate = 1;
-				}
-			}
 			push @paragraph, @text;
 		}
 
@@ -1192,8 +1335,10 @@ sub treat_content {
 	}
 
 	# This strips the extracted strings
-	# (only if you don't specify the 'nostrip' option)
-	if (!$self->{options}{'nostrip'}) {
+	# (only if you don't specify the 'nostrip' option, and if the
+	# paragraph can be re-wrapped)
+	$translate = $self->get_translate_options($self->get_path);
+	if (!$self->{options}{'nostrip'} and $translate !~ m/W/) {
 		my $clean = 0;
 		# Clean the beginning
 		while (!$clean and $#paragraph > 0) {
@@ -1293,36 +1438,66 @@ sub treat_content {
 # The $translate argument indicates if the strings must be translated or
 # just pushed
 sub translate_paragraph {
+# TODO: remove the translate parameter
 	my ($self, $translate) = (shift, shift);
 	my @paragraph = @_;
+	$translate = $self->get_translate_options($self->get_path);
 
 	my $comments;
 	while (@comments) {
-		my ($t,$l) = (shift @comments, shift @comments);
+		my ($comment,$eoc);
+		do {
+			my ($t,$l) = (shift @comments, shift @comments);
+			$t =~ s/\n?(\0)?$//;
+			$eoc = $1;
+			$comment .= "\n" if defined $comment;
+			$comment .= $t;
+		} until ($eoc);
 		$comments .= "\n" if defined $comments;
-		$comments .= $t;
+		$comments .= $comment;
+		$self->pushline("<!--".$comment."-->\n") if defined $comment;
 	}
-	$self->pushline("<!--".$comments."-->\n") if defined $comments;
 	@comments = ();
 
-	if ( length($self->join_lines(@paragraph)) > 0 ) {
-		my $struc = $self->get_path;
-		my $options = $self->tag_in_list($struc,@{$self->{tags}});
-		$options = "" if ($options eq 0 or $options eq 1);
-		if ($translate) {
+	if ($self->{options}{'cpp'}) {
+		my @tmp = @paragraph;
+		@paragraph = ();
+		while (@tmp) {
+			my ($t,$l) = (shift @tmp, shift @tmp);
+			# #include can be followed by a filename between
+			# <> brackets. In that case, the argument won't be
+			# handled in the same call to translate_paragraph.
+			# Thus do not try to match "include ".
+			if ($t =~ m/^#[ \t]*(if |endif|undef |include|else|ifdef |ifndef |define )/si) {
+				if (@paragraph) {
+					$self->translate_paragraph($translate,
+					                           @paragraph);
+					@paragraph = ();
+					$self->pushline("\n");
+				}
+				$self->pushline($t);
+			} else {
+				push @paragraph, ($t,$l);
+			}
+		}
+	}
+
+	my $para = $self->join_lines(@paragraph);
+	if ( length($para) > 0 ) {
+		if ($translate ne "") {
 			# This tag should be translated
 			$self->pushline($self->found_string(
-				$self->join_lines(@paragraph),
+				$para,
 				$paragraph[1], {
 					type=>"tag",
-					tag_options=>$options,
+					tag_options=>$translate,
 					comments=>$comments
 				}));
 		} else {
 			# Inform that this tag isn't translated in debug mode
-			print wrap_ref_mod($paragraph[1], "po4a::xml", dgettext ("po4a", "Content of tag %s excluded: %s"), $self->get_path, $self->join_lines(@paragraph))
+			print wrap_ref_mod($paragraph[1], "po4a::xml", dgettext ("po4a", "Content of tag %s excluded: %s"), $self->get_path, $para)
 			       if $self->debug();
-			$self->pushline($self->recode_skipped_text($self->join_lines(@paragraph)));
+			$self->pushline($self->recode_skipped_text($para));
 		}
 	}
 }
@@ -1345,22 +1520,6 @@ or in the initialize function).
 
 sub treat_options {
 	my $self = shift;
-
-	$self->{options}{'tags'} =~ /\s*(.*)\s*/s;
-	my @list_tags = split(/\s+/s,$1);
-	$self->{tags} = \@list_tags;
-
-	$self->{options}{'attributes'} =~ /\s*(.*)\s*/s;
-	my @list_attr = split(/\s+/s,$1);
-	$self->{attributes} = \@list_attr;
-
-	$self->{options}{'inline'} =~ /\s*(.*)\s*/s;
-	my @list_inline = split(/\s+/s,$1);
-	$self->{inline} = \@list_inline;
-
-	$self->{options}{'placeholder'} =~ /\s*(.*)\s*/s;
-	my @list_placeholder = split(/\s+/s,$1);
-	$self->{placeholder} = \@list_placeholder;
         
 	$self->{options}{'nodefault'} =~ /\s*(.*)\s*/s;
 	my %list_nodefault;
@@ -1368,6 +1527,38 @@ sub treat_options {
 	    $list_nodefault{$_} = 1;
 	}
 	$self->{nodefault} = \%list_nodefault;
+
+	$self->{options}{'tags'} =~ /\s*(.*)\s*/s;
+	my @list_tags;
+	foreach my $tag (split(/\s+/s,$1)) {
+		push @list_tags, $tag
+			unless $list_nodefault{$tag};
+	}
+	$self->{tags} = \@list_tags;
+
+	$self->{options}{'translated'} =~ /\s*(.*)\s*/s;
+	my @list_translated = split(/\s+/s,$1);
+	$self->{translated} = \@list_translated;
+
+	$self->{options}{'untranslated'} =~ /\s*(.*)\s*/s;
+	my @list_untranslated = split(/\s+/s,$1);
+	$self->{untranslated} = \@list_untranslated;
+
+	$self->{options}{'attributes'} =~ /\s*(.*)\s*/s;
+	my @list_attr = split(/\s+/s,$1);
+	$self->{attributes} = \@list_attr;
+
+	$self->{options}{'inline'} =~ /\s*(.*)\s*/s;
+	my @list_inline;
+	foreach my $tag (split(/\s+/s,$1)) {
+		push @list_inline, $tag
+			unless $list_nodefault{$tag};
+	}
+	$self->{inline} = \@list_inline;
+
+	$self->{options}{'placeholder'} =~ /\s*(.*)\s*/s;
+	my @list_placeholder = split(/\s+/s,$1);
+	$self->{placeholder} = \@list_placeholder;
 }
 
 =head2 GETTING TEXT FROM THE INPUT DOCUMENT
